@@ -2,13 +2,15 @@ import traceback
 import json
 import time
 import logging
-from functools import partial
+import math
 from copy import deepcopy
-from tqdm import tqdm
-from multiprocessing import Process, Manager, Queue, Pool, Array
+from tqdm.autonotebook import tqdm
+from multiprocessing import Process, Manager, Queue, cpu_count
 from time import sleep
+from typing import Union, List
+from collections.abc import Iterable
+from spacy.tokens import Span
 
-from medcat.cdb import CDB
 from medcat.preprocessing.tokenizers import spacy_split_all
 from medcat.pipe import Pipe
 from medcat.preprocessing.taggers import tag_skip_and_punct
@@ -44,7 +46,7 @@ class CAT(object):
             Concept database used with this CAT instance, please do not assign
             this value directly.
         config (medcat.config.Config):
-            The global configuration for medcat. Usuall cdb.config can be used for this
+            The global configuration for medcat. Usually cdb.config can be used for this
             field.
         vocab (medcat.utils.vocab.Vocab):
             The vocabulary object used with this instance, please do not assign
@@ -56,6 +58,7 @@ class CAT(object):
         >>>spacy_doc = cat("Put some text here")
         >>>print(spacy_doc.ents) # Detected entites
     '''
+    DEFAULT_MAX_DOCUMENT_LENGTH = 1000000
     log = logging.getLogger(__package__)
     # Add file and console handlers
     log = add_handlers(log)
@@ -70,7 +73,7 @@ class CAT(object):
 
         # Build the pipeline
         self.nlp = Pipe(tokenizer=spacy_split_all, config=self.config)
-        self.nlp.add_tagger(tagger=partial(tag_skip_and_punct, config=self.config),
+        self.nlp.add_tagger(tagger=tag_skip_and_punct,
                             name='skip_and_punct',
                             additional_fields=['is_punct'])
 
@@ -92,7 +95,7 @@ class CAT(object):
             self._meta_annotations = True
 
         # Set max document length
-        self.nlp.nlp.max_length = self.config.preprocessing.get('max_document_length', 1000000)
+        self.nlp.nlp.max_length = self.config.preprocessing.get('max_document_length', CAT.DEFAULT_MAX_DOCUMENT_LENGTH)
 
 
     def get_spacy_nlp(self):
@@ -100,31 +103,41 @@ class CAT(object):
         '''
         return self.nlp.nlp
 
-
-    def __call__(self, text, do_train=False):
+    def __call__(self, text: Union[str, List[str]], do_train: bool = False):
         r'''
         Push the text through the pipeline.
 
         Args:
-            text (string):
-                The text to be annotated, if it is longer than self.config.preprocessing['max_document_length'] it will be trimmed
-                to that length.
+            text (string/List):
+                The text or the list of texts to be annotated, if the text length is longer than
+                self.config.preprocessing['max_document_length'] it will be trimmed to that length.
             do_train (bool, defaults to `False`):
                 This causes so many screwups when not there, so I'll force training
                 to False. To run training it is much better to use the self.train() function
                 but for some special cases I'm leaving it here also.
         Returns:
-            A spacy document with the extracted entities
+            A single spacy document or multiple spacy documents with the extracted entities
         '''
-        # Should we train - do not use this for training, unles you know what you are doing. Use the
+        # Should we train - do not use this for training, unless you know what you are doing. Use the
         #self.train() function
         self.config.linking['train'] = do_train
 
-        if text and len(text) > 0:
-            return self.nlp(text[0:self.config.preprocessing.get('max_document_length', 1000000)])
+        if isinstance(text, str):
+            if text and len(text) > 0:
+                return self.nlp(text[0:self.config.preprocessing.get('max_document_length', CAT.DEFAULT_MAX_DOCUMENT_LENGTH)])
+            else:
+                return None
+                # return self.nlp("")     # For conformity, previously returning None
+        elif isinstance(text, list):
+            truncated = []
+            for t in text:
+                if isinstance(t, str) and len(t) > 0:
+                    truncated.append(t[0:self.config.preprocessing.get('max_document_length', CAT.DEFAULT_MAX_DOCUMENT_LENGTH)])
+                else:
+                    truncated.append("")
+            return self.nlp(truncated)
         else:
-            return None
-
+            raise ValueError("The input text should be either a string or a list of strings")
 
     def _print_stats(self, data, epoch=0, use_filters=False, use_overlaps=False, use_cui_doc_limit=False,
                      use_groups=False):
@@ -164,7 +177,7 @@ class CAT(object):
             cui_counts (dict):
                 Number of occurrence for each CUI
             examples (dict):
-                Examples for each of the fp, fn, tp. Foramt will be examples['fp']['cui'][<list_of_examples>]
+                Examples for each of the fp, fn, tp. Format will be examples['fp']['cui'][<list_of_examples>]
         '''
         tp = 0
         fp = 0
@@ -197,10 +210,7 @@ class CAT(object):
 
             start_time = time.time()
             for dind, doc in tqdm(enumerate(project['documents']), desc='Stats document', total=len(project['documents']), leave=False):
-                if type(doc['annotations']) == list:
-                    anns = doc['annotations']
-                elif type(doc['annotations']) == dict:
-                    anns = doc['annotations'].values()
+                anns = self._get_doc_annotations(doc)
 
                 # Apply document level filtering if
                 if use_cui_doc_limit:
@@ -340,9 +350,8 @@ class CAT(object):
 
         return fps, fns, tps, cui_prec, cui_rec, cui_f1, cui_counts, examples
 
-
     def train(self, data_iterator, fine_tune=True, progress_print=1000):
-        """ Runs training on the data, note that the maximum lenght of a line
+        """ Runs training on the data, note that the maximum length of a line
         or document is 1M characters. Anything longer will be trimmed.
 
         data_iterator:
@@ -374,7 +383,6 @@ class CAT(object):
 
         self.config.linking['train'] = False
 
-
     def add_cui_to_group(self, cui, group_name, reset_all_groups=False):
         r'''
         Ads a CUI to a group, will appear in cdb.addl_info['cui2group']
@@ -398,7 +406,6 @@ class CAT(object):
         # Add group_name
         self.cdb.addl_info['cui2group'][cui] = group_name
 
-
     def unlink_concept_name(self, cui, name, preprocessed_name=False):
         r'''
         Unlink a concept name from the CUI (or all CUIs if full_unlink), removes the link from
@@ -430,10 +437,9 @@ class CAT(object):
         for cui in cuis:
             self.cdb.remove_names(cui=cui, names=names)
 
-
     def add_and_train_concept(self, cui, name, spacy_doc=None, spacy_entity=None, ontologies=set(), name_status='A', type_ids=set(),
                               description='', full_build=True, negative=False, devalue_others=False, do_add_concept=True):
-        r''' Add a name to an existing concept, or add a new concept, or do not do anything if the name and concept alraedy exist. Perform
+        r''' Add a name to an existing concept, or add a new concept, or do not do anything if the name or concept already exists. Perform
         training if spacy_entity and spacy_doc are set.
 
         Args:
@@ -471,19 +477,18 @@ class CAT(object):
                 for name in names:
                     cuis.update(self.cdb.name2cuis.get(name, []))
                 # Remove the cui for which we just added positive training
-                cuis.remove(cui)
+                if cui in cuis:
+                    cuis.remove(cui)
                 # Add negative training for all other CUIs that link to these names
                 for _cui in cuis:
                     self.linker.context_model.train(cui=_cui, entity=spacy_entity, doc=spacy_doc, negative=True)
 
-
-
     def train_supervised(self, data_path, reset_cui_count=False, nepochs=1,
                          print_stats=0, use_filters=False, terminate_last=False, use_overlaps=False,
                          use_cui_doc_limit=False, test_size=0, devalue_others=False, use_groups=False,
                          never_terminate=False, train_from_false_positives=False):
         r''' TODO: Refactor, left from old
-        Run supervised training on a dataset from MedCATtrainer. Please take care that this is more a simiulated
+        Run supervised training on a dataset from MedCATtrainer. Please take care that this is more a simulated
         online training then supervised.
 
         Args:
@@ -492,7 +497,7 @@ class CAT(object):
             reset_cui_count (boolean):
                 Used for training with weight_decay (annealing). Each concept has a count that is there
                 from the beginning of the CDB, that count is used for annealing. Resetting the count will
-                significantly incrase the training impact. This will reset the count only for concepts
+                significantly increase the training impact. This will reset the count only for concepts
                 that exist in the the training data.
             nepochs (int):
                 Number of epochs for which to run the training.
@@ -504,7 +509,7 @@ class CAT(object):
             terminate_last (boolean):
                 If true, concept termination will be done after all training.
             use_overlaps (boolean):
-                Allow overlapping entites, nearly always False as it is very difficult to annotate overlapping entites.
+                Allow overlapping entities, nearly always False as it is very difficult to annotate overlapping entities.
             use_cui_doc_limit (boolean):
                 If True the metrics for a CUI will be only calculated if that CUI appears in a document, in other words
                 if the document was annotated for that CUI. Useful in very specific situations when during the annotation
@@ -540,7 +545,8 @@ class CAT(object):
                 FP/FN examples of sentences for each CUI
         '''
         fp = fn = tp = p = r = f1 = cui_counts = examples = {}
-        data = json.load(open(data_path))
+        with open(data_path) as f:
+            data = json.load(f)
         cui_counts = {}
 
         if test_size == 0:
@@ -559,11 +565,7 @@ class CAT(object):
             cuis = []
             for project in train_set['projects']:
                 for doc in project['documents']:
-                    if type(doc['annotations']) == list:
-                        doc_annotations = doc['annotations']
-                    elif type(doc['annotations']) == dict:
-                        doc_annotations = doc['annotations'].values()
-
+                    doc_annotations = self._get_doc_annotations(doc)
                     for ann in doc_annotations:
                         cuis.append(ann['cui'])
             for cui in set(cuis):
@@ -574,11 +576,7 @@ class CAT(object):
         if not never_terminate:
             for project in train_set['projects']:
                 for doc in project['documents']:
-                    if type(doc['annotations']) == list:
-                        doc_annotations = doc['annotations']
-                    elif type(doc['annotations']) == dict:
-                        doc_annotations = doc['annotations'].values()
-
+                    doc_annotations = self._get_doc_annotations(doc)
                     for ann in doc_annotations:
                         if ann.get('killed', False):
                             self.unlink_concept_name(ann['cui'], ann['value'])
@@ -589,11 +587,7 @@ class CAT(object):
                 for i_doc, doc in tqdm(enumerate(project['documents']), desc='Document', leave=False, total=len(project['documents'])):
                     spacy_doc = self(doc['text'])
                     # Compatibility with old output where annotations are a list
-                    if type(doc['annotations']) == list:
-                        doc_annotations = doc['annotations']
-                    elif type(doc['annotations']) == dict:
-                        doc_annotations = doc['annotations'].values()
-
+                    doc_annotations = self._get_doc_annotations(doc)
                     for ann in doc_annotations:
                         if not ann.get('killed', False):
                             cui = ann['cui']
@@ -622,11 +616,7 @@ class CAT(object):
                 # Remove entities that were terminated, but after all training is done
                 for project in train_set['projects']:
                     for doc in project['documents']:
-                        if type(doc['annotations']) == list:
-                            doc_annotations = doc['annotations']
-                        elif type(doc['annotations']) == dict:
-                            doc_annotations = doc['annotations'].values()
-
+                        doc_annotations = self._get_doc_annotations(doc)
                         for ann in doc_annotations:
                             if ann.get('killed', False):
                                 self.unlink_concept_name(ann['cui'], ann['value'])
@@ -639,74 +629,25 @@ class CAT(object):
                                                          use_groups=use_groups)
         return fp, fn, tp, p, r, f1, cui_counts, examples
 
-
-    def get_entities(self, text, only_cui=False, addl_info=['cui2icd10', 'cui2ontologies', 'cui2snomed']):
+    def get_entities(self, text, only_cui=False, addl_info=['cui2icd10', 'cui2ontologies', 'cui2snomed'], n_process=None, batch_size=None):
         r''' Get entities
 
         text:  text to be annotated
         return:  entities
         '''
         cnf_annotation_output = getattr(self.config, 'annotation_output', {})
-        doc = self(text)
-        out = {'entities': {}, 'tokens': []}
-        if doc is not None:
-            out_ent = {}
-            if self.config.general.get('show_nested_entities', False):
-                _ents = doc._.ents
-            else:
-                _ents = doc.ents
-
-            if cnf_annotation_output.get("lowercase_context", True):
-                doc_tokens = [tkn.text_with_ws.lower() for tkn in list(doc)]
-            else:
-                doc_tokens = [tkn.text_with_ws for tkn in list(doc)]
-
-            if cnf_annotation_output.get('doc_extended_info', False):
-                # Add tokens if extended info
-                out['tokens'] = doc_tokens
-
-            context_left = cnf_annotation_output.get('context_left', -1)
-            context_right = cnf_annotation_output.get('context_right', -1)
-            doc_extended_info = cnf_annotation_output.get('doc_extended_info', False)
-
-            for ind, ent in enumerate(_ents):
-                cui = str(ent._.cui)
-                if not only_cui:
-                    out_ent['pretty_name'] = self.cdb.cui2preferred_name.get(cui, '')
-                    out_ent['cui'] = cui
-                    out_ent['tuis'] = list(self.cdb.cui2type_ids.get(cui, ''))
-                    out_ent['types'] = [self.cdb.addl_info['type_id2name'].get(tui, '') for tui in out_ent['tuis']]
-                    out_ent['source_value'] = ent.text
-                    out_ent['detected_name'] = str(ent._.detected_name)
-                    out_ent['acc'] = float(ent._.context_similarity)
-                    out_ent['context_similarity'] = float(ent._.context_similarity)
-                    out_ent['start'] = ent.start_char
-                    out_ent['end'] = ent.end_char
-                    for addl in addl_info:
-                        tmp = self.cdb.addl_info.get(addl, {}).get(cui, [])
-                        out_ent[addl.split("2")[-1]] = list(tmp) if type(tmp) == set else tmp
-                    out_ent['id'] = ent._.id
-                    out_ent['meta_anns'] = {}
-
-                    if doc_extended_info:
-                        out_ent['start_tkn'] = ent.start
-                        out_ent['end_tkn'] = ent.end
-
-                    if context_left > 0 and context_right > 0:
-                        out_ent['context_left'] = doc_tokens[max(ent.start - context_left, 0):ent.start]
-                        out_ent['context_right'] = doc_tokens[ent.end:min(ent.end + context_right, len(doc_tokens))]
-                        out_ent['context_center'] = doc_tokens[ent.start:ent.end]
-
-                    if hasattr(ent._, 'meta_anns') and ent._.meta_anns:
-                        out_ent['meta_anns'] = ent._.meta_anns
-
-                    out['entities'][out_ent['id']] = dict(out_ent)
-                else:
-                    out['entities'][ent._.id] = cui
-
+        if isinstance(text, str):
+            doc = self(text)
+            out = self._doc_to_out(doc, cnf_annotation_output, only_cui, addl_info)
+        elif isinstance(text, Iterable):
+            out = []
+            docs = self.nlp.batch_multi_process(text, n_process, batch_size)
+            for doc in docs:
+                out.append(self._doc_to_out(doc, cnf_annotation_output, only_cui, addl_info))
+        else:
+            raise ValueError("The input text should be either a string or a list of strings")
         return out
 
-
     def get_json(self, text, only_cui=False, addl_info=['cui2icd10', 'cui2ontologies']):
         """ Get output in json format
 
@@ -718,7 +659,6 @@ class CAT(object):
 
         return json.dumps(out)
 
-
     def multiprocessing(self, in_data, nproc=8, batch_size_chars=1000000, only_cui=False, addl_info=[]):
         r''' Run multiprocessing NOT FOR TRAINING
 
@@ -752,7 +692,7 @@ class CAT(object):
         for id, text in in_data:
             data.append((id, str(text)))
             nchars += len(str(text))
-            if  nchars >= batch_size_chars:
+            if nchars >= batch_size_chars:
                 in_q.put(data)
                 data = []
                 nchars = 0
@@ -781,6 +721,38 @@ class CAT(object):
 
         return out
 
+    def multiprocessing_pipe(self, in_data, nproc=None, batch_size=None, only_cui=False, addl_info=[]):
+        r''' Run multiprocessing NOT FOR TRAINING
+
+        in_data:  an iterator or array with format: [(id, text), (id, text), ...]
+        nproc:  the number of processors
+        batch_size: the number of texts to buffer
+
+        return:  an list of tuples: [(id, doc_json), (id, doc_json), ...]
+        '''
+        def _generate_texts(data):
+            for _, text in data:
+                if isinstance(text, str) and len(text) > 0:
+                    yield text[0:self.config.preprocessing.get('max_document_length', CAT.DEFAULT_MAX_DOCUMENT_LENGTH)]
+                else:
+                    yield ""
+
+        if self._meta_annotations:
+            # Hack for torch using multithreading, which is not good here
+            import torch
+            torch.set_num_threads(1)
+
+        n_process = nproc if nproc is not None else max(cpu_count() - 1, 1)
+        batch_size = batch_size if batch_size is not None else math.ceil(len(in_data) / n_process)
+
+        entities = self.get_entities(text=_generate_texts(in_data), only_cui=only_cui, addl_info=addl_info,
+                                     n_process=n_process, batch_size=batch_size)
+        out = []
+        for idx in range(len(in_data)):
+            entities[idx]['text'] = in_data[idx][1]
+            out.append((in_data[idx][0], entities[idx]))
+
+        return out
 
     def _mp_cons(self, in_q, out_dict, pid=0, only_cui=False, addl_info=[]):
         cnt = 0
@@ -803,3 +775,82 @@ class CAT(object):
                         self.log.warning(e, stack_info=True)
 
             sleep(1)
+
+    def _doc_to_out(self, doc, cnf_annotation_output, only_cui, addl_info):
+        out = {'entities': {}, 'tokens': []}
+        if doc is not None:
+            out_ent = {}
+            if self.config.general.get('show_nested_entities', False):
+                _ents = []
+                for _ent in doc._.ents:
+                    entity = Span(doc, _ent['start'], _ent['end'], label=_ent['label'])
+                    entity._.cui = _ent['cui']
+                    entity._.detected_name = _ent['detected_name']
+                    entity._.context_similarity = _ent['context_similarity']
+                    entity._.id = _ent['id']
+                    if 'meta_anns' in _ent:
+                        entity._.meta_anns = _ent['meta_anns']
+                    _ents.append(entity)
+            else:
+                _ents = doc.ents
+
+            if cnf_annotation_output.get("lowercase_context", True):
+                doc_tokens = [tkn.text_with_ws.lower() for tkn in list(doc)]
+            else:
+                doc_tokens = [tkn.text_with_ws for tkn in list(doc)]
+
+            if cnf_annotation_output.get('doc_extended_info', False):
+                # Add tokens if extended info
+                out['tokens'] = doc_tokens
+
+            context_left = cnf_annotation_output.get('context_left', -1)
+            context_right = cnf_annotation_output.get('context_right', -1)
+            doc_extended_info = cnf_annotation_output.get('doc_extended_info', False)
+
+            for ind, ent in enumerate(_ents):
+                cui = str(ent._.cui)
+                if not only_cui:
+                    out_ent['pretty_name'] = self.cdb.get_name(cui)
+                    out_ent['cui'] = cui
+                    out_ent['tuis'] = list(self.cdb.cui2type_ids.get(cui, ''))
+                    out_ent['types'] = [self.cdb.addl_info['type_id2name'].get(tui, '') for tui in out_ent['tuis']]
+                    out_ent['source_value'] = ent.text
+                    out_ent['detected_name'] = str(ent._.detected_name)
+                    out_ent['acc'] = float(ent._.context_similarity)
+                    out_ent['context_similarity'] = float(ent._.context_similarity)
+                    out_ent['start'] = ent.start_char
+                    out_ent['end'] = ent.end_char
+                    for addl in addl_info:
+                        tmp = self.cdb.addl_info.get(addl, {}).get(cui, [])
+                        out_ent[addl.split("2")[-1]] = list(tmp) if type(tmp) == set else tmp
+                    out_ent['id'] = ent._.id
+                    out_ent['meta_anns'] = {}
+
+                    if doc_extended_info:
+                        out_ent['start_tkn'] = ent.start
+                        out_ent['end_tkn'] = ent.end
+
+                    if context_left > 0 and context_right > 0:
+                        out_ent['context_left'] = doc_tokens[max(ent.start - context_left, 0):ent.start]
+                        out_ent['context_right'] = doc_tokens[ent.end:min(ent.end + context_right, len(doc_tokens))]
+                        out_ent['context_center'] = doc_tokens[ent.start:ent.end]
+
+                    if hasattr(ent._, 'meta_anns') and ent._.meta_anns:
+                        out_ent['meta_anns'] = ent._.meta_anns
+
+                    out['entities'][out_ent['id']] = dict(out_ent)
+                else:
+                    out['entities'][ent._.id] = cui
+        return out
+
+    @staticmethod
+    def _get_doc_annotations(doc):
+        if type(doc['annotations']) == list:
+            return doc['annotations']
+        elif type(doc['annotations']) == dict:
+            return doc['annotations'].values()
+        else:
+            return None
+
+    def destroy_pipe(self):
+        self.nlp.destroy()
