@@ -900,6 +900,7 @@ def prepare_from_json(data, cntx_left, cntx_right, tokenizer, cntx_in_chars=Fals
                             if meta_ann not in ann['meta_anns'] or ann['meta_anns'][meta_ann]['value'] != prerequisite[meta_ann]:
                                 # Skip this annotation as the prerequisite is not met
                                 skip = True
+                                break
 
                     if not skip and (cui_filter is None or not cui_filter or cui in cui_filter):
                         if ann.get('validated', True) and (not ann.get('deleted', False) and not ann.get('killed', False)):
@@ -989,8 +990,11 @@ def make_mc_train_test(data, cdb, seed=17, test_size=0.2):
 
         if 'cuis' in project and len(project['cuis'].strip()) > 0:
             cui_filter = [x.strip() for x in project['cuis'].split(",")]
+
+        """
         if 'tuis' in project and len(project['tuis'].strip()) > 0:
             tui_filter = [x.strip().upper() for x in project['tuis'].split(",")]
+        """
 
         for document in project['documents']:
             if type(document['annotations']) == list:
@@ -999,9 +1003,7 @@ def make_mc_train_test(data, cdb, seed=17, test_size=0.2):
                 doc_annotations = document['annotations'].values()
 
             for ann in doc_annotations:
-
-                if (cui_filter is None and tui_filter is None) or (cui_filter is not None and ann['cui'] in cui_filter) or \
-                   (tui_filter is not None and cdb.cui2tui.get(ann['cui'], 'unk') in tui_filter):
+                if (cui_filter is None and tui_filter is None) or (cui_filter is not None and ann['cui'] in cui_filter):
                     if ann['cui'] in cnts:
                         cnts[ann['cui']] += 1
                     else:
@@ -1034,8 +1036,10 @@ def make_mc_train_test(data, cdb, seed=17, test_size=0.2):
 
         if 'cuis' in project and len(project['cuis'].strip()) > 0:
             cui_filter = [x.strip() for x in project['cuis'].split(",")]
+        """
         if 'tuis' in project and len(project['tuis'].strip()) > 0:
             tui_filter = [x.strip().upper() for x in project['tuis'].split(",")]
+        """
 
 
         for i_document in np.random.permutation(np.arange(0, len(project['documents']))):
