@@ -139,6 +139,15 @@ def tkn_inds_from_doc(spacy_doc, text_inds=None, source_val=None):
     return tkn_inds
 
 
+def tkns_from_doc(spacy_doc, start, end):
+    tkns = []
+    for tkn in spacy_doc:
+        if tkn.idx >= start and tkn.idx <= end:
+            tkns.append(tkn)
+
+    return tkns
+
+
 def filter_cdb_by_icd10(cdb: CDB) -> CDB:
     """
     Filters an existing CDB to only contain concepts that have an associated ICD-10 code.
@@ -389,20 +398,19 @@ def remove_icd10_ranges(cdb):
                 del cdb.cui2info[cui]['icd10']
 
 
-def check_scispacy():
+def dep_check_scispacy():
+    # IGNORE FUNCTION
     import spacy
     import subprocess
     import sys
     try:
-        nlp = spacy.load("en_core_sci_md")
+        _ = spacy.load("en_core_sci_md")
     except:
         print("Installing the missing models for scispacy\n")
-        pkg = 'https://s3-us-west-2.amazonaws.com/ai2-s2-scispacy/releases/v0.2.4/en_core_sci_md-0.2.4.tar.gz'
+        pkg = 'https://s3-us-west-2.amazonaws.com/ai2-s2-scispacy/releases/v0.4.0/en_core_sci_md-0.4.0.tar.gz'
         subprocess.check_call([sys.executable, '-m', 'pip', 'install', pkg])
 
 
-
-
 def run_cv(cdb_path, data_path, vocab_path, cv=100, nepochs=16, test_size=0.1, lr=1, groups=None, **kwargs):
     from medcat.cat import CAT
     from medcat.utils.vocab import Vocab
