@@ -1,9 +1,6 @@
-#import hunspell
 import re
-from collections import Counter
-from spacy.tokens import Span
 import spacy
-import os
+
 
 CONTAINS_NUMBER = re.compile('[0-9]+')
 
@@ -90,6 +87,9 @@ class TokenNormalizer(object):
         spell_checker
     '''
 
+    # Custom pipeline component name
+    name = 'token_normalizer'
+
     def __init__(self, config, spell_checker=None):
         self.config = config
         self.spell_checker = spell_checker
